using System;

public delegate void StringDelegate(string aValue); 

public class TestDelegateClass
{     
	void PrintString(string aValue)
	{
		Console.WriteLine(aValue);
	}

	void PrintStringLength(string aValue)
	{
		Console.WriteLine("Length = {0}", aValue.Length);
	}

	static void PrintStringWithDate(string aValue)
	{
		Console.WriteLine("{0}: {1}", DateTime.Now, aValue);
	}

	static void PrintInvocationList(Delegate aDelegate) 
	{
		Console.Write("(");
		Delegate[] list = aDelegate.GetInvocationList();
		foreach (Delegate d in list)
		{
			Console.Write(" {0}", d.Method.Name);
		}
		Console.WriteLine(" )");
	}

	public static void Main(String[] args)
	{
		TestDelegateClass tdc = new TestDelegateClass();               
		StringDelegate printDelegate =
			new StringDelegate(tdc.PrintString);
		StringDelegate printLengthDelegate =
			new StringDelegate(tdc.PrintStringLength);
		StringDelegate printWithDateDelegate =
			new StringDelegate(PrintStringWithDate);

		PrintInvocationList(printDelegate);
		// Prints: ( PrintString )

		StringDelegate combinedDelegate = (StringDelegate)
			Delegate.Combine(printDelegate,
			printLengthDelegate);

		PrintInvocationList(combinedDelegate);
		// Prints: ( PrintString PrintStringLength )

		combinedDelegate = (StringDelegate)
			Delegate.Combine(combinedDelegate,
			printWithDateDelegate);

		PrintInvocationList(combinedDelegate);
		// Prints: ( PrintString PrintStringLength PrintStringWithDate )

		// Invoke the delegate
		combinedDelegate("test");
	}
}